﻿{ *********************************************************************** }
{                                                                         }
{ Adit Math Engine                                                        }
{ Math Images Definition                                                  }
{                                                                         }
{ Copyright (c) 2015-21 Adit Software http://www.aditmath.com/            }
{                                                                         }
{ *********************************************************************** }

unit MathImages;

interface

uses
  Winapi.Windows, Winapi.Messages,
  System.SysUtils, System.Variants, System.Classes;

type
  TMathSymbolType = (
    mstBasic,
    mstGreekLower,
    mstGreekUpper,
    mstLetterLike,
    mstBinaryOperators,
    mstRelationOperators,
    mstNaryOperators,
    mstAdvBinaryOperators,
    mstAdvRelationOperators,
    mstArrows,
    mstNegativeRelations,
    mstScripts,
    mstFrankturs,
    mstDoubleStruck,
    mstGeometry);

  TMathExpressionType = (
    metFraction,
    metCommonFraction,
    metScript,
    metCommonScript,
    metRadical,
    metCommonRadical,
    metIntegral,
    metContourIntegral,
    metDifferential,
    metSumms,
    metProds,
    metUnions,
    metOtherLarge,
    metCommonLarge,
    metBracket,
    metBracketWithSeparator,
    metSingleBracket,
    metCaseAndStack,
    metCommonBracket,
    metTrigFunction,
    metInvFunction,
    metHyperFunction,
    metInvHyperFunction,
    metCommonFunction,
    metAccent,
    metBoxed,
    metOverUnderbar,
    metCommonAccent,
    metLimitAndLog,
    metCommonLimitAndLog,
    metOperatorBasic,
    metOperatorStruct,
    metCommonOperator,
    metEmptyMatrix,
    metDots,
    metIndentityMatrix,
    metMatrixWithBrackets,
    metSparseMatrix
  );

  TMathExpressionImageSize = (
    meizNormal,
    meizNormalMedium,
    meizNormalWide,
    meizNormalSuperWide,
    meizSmall,
    meizSmallWide
  );

  TMathSymbolRecord = record
    Command: string;
    Hint: string;
  end;

  TMathSymbolRanges = record
    SymbType: TMathSymbolType;
    ImageIndexes: array[0..73] of integer;
  end;

  TMathExpressionRecord = record
    Size: TMathExpressionImageSize;
    ExprType: TMathExpressionType;
    Command: string;
    Hint: string;
  end;

const
  MathSymbols: array[0..527] of TMathSymbolRecord = (
    (Command: '\pm';                 Hint: 'Plus Minus'),
    (Command: '\infty';              Hint: 'Infinity'),
    (Command: '=';                   Hint: 'Equal'),
    (Command: '\ne';                 Hint: 'Not Equal To'),
    (Command: '\sim';                Hint: 'Approximately'),
    (Command: '\times';              Hint: 'Multiplication Sign'),
    (Command: '\slash';              Hint: 'Division Sign'),
    (Command: '!';                   Hint: 'Factorial'),
    (Command: '\propto';             Hint: 'Proportional To'),
    (Command: '<';                   Hint: 'Less Than'),
    (Command: '\ll';                 Hint: 'Much Less Than'),
    (Command: '>';                   Hint: 'Greater Than'),
    (Command: '\gg';                 Hint: 'Much Greater Than'),
    (Command: '\leq';                Hint: 'Less Than or Equal To'),
    (Command: '\geq';                Hint: 'Greater Than or Equal To'),
    (Command: '\mp';                 Hint: 'Minus Plus'),
    (Command: '\cong';               Hint: 'Approximately Equal To'),
    (Command: '\approx';             Hint: 'Almost Equal To'),
    (Command: '\equiv';              Hint: 'Identical To'),
    (Command: '\forall';             Hint: 'For All'),
    (Command: '\complement';         Hint: 'Complement'),
    (Command: '\partial';            Hint: 'Partial Differential'),
    (Command: '\surd';               Hint: 'Radical Sign'),
    (Command: '\cuberoot';           Hint: 'Cube Root'),
    (Command: '\fourthroot';         Hint: 'Fourth Root'),
    (Command: '\cup';                Hint: 'Union'),
    (Command: '\cap';                Hint: 'Intersection'),
    (Command: '\emptyset';           Hint: 'Empty Set'),
    (Command: '\percent';            Hint: 'Percentage'),
    (Command: '\degree';             Hint: 'Degrees'),
    (Command: '\fahrenheit';         Hint: 'Degrees Fahrenheit'),
    (Command: '\celsius';            Hint: 'Degrees Celsius'),
    (Command: '\nabla';              Hint: 'Nable'),
    (Command: '\exists';             Hint: 'There Exists'),
    (Command: '\nexists';            Hint: 'There Does Not Exists'),
    (Command: '\in';                 Hint: 'Element Of'),
    (Command: '\ni';                 Hint: 'Contains as Member'),
    (Command: '\leftarrow';          Hint: 'Left Arrow'),
    (Command: '\uparrow';            Hint: 'Up Arrow'),
    (Command: '\to';                 Hint: 'Right Arrow'),
    (Command: '\downarrow';          Hint: 'Down Arrow'),
    (Command: '\leftrightarrow';     Hint: 'Left-Right Arrow'),
    (Command: '\therefore';          Hint: 'Therefore'),
    (Command: '+';                   Hint: 'Plus'),
    (Command: '-';                   Hint: 'Minus'),
    (Command: '\neg';                Hint: 'Not Sign'),
    (Command: '\ast';                Hint: 'Asterisk Operator'),
    (Command: '\bullet';             Hint: 'Buller Operator'),
    (Command: '\vdots';              Hint: 'Vertical Ellipsis'),
    (Command: '\cdots';              Hint: 'Midline Horizontal Ellipsis'),
    (Command: '\adots';              Hint: 'Up Right Diagonal Ellipsis'),
    (Command: '\ddots';              Hint: 'Down Right Diagonal Ellipsis'),
    (Command: '\aleph';              Hint: 'Alef'),
    (Command: '\beth';               Hint: 'Bet'),
    (Command: '\QED';                Hint: 'End of Proof'),
    (Command: '\alpha';              Hint: 'Alpha'),
    (Command: '\beta';               Hint: 'Beta'),
    (Command: '\gamma';              Hint: 'Gamma'),
    (Command: '\delta';              Hint: 'Delta'),
    (Command: '\varepsilon';         Hint: 'Epsilon Variant'),
    (Command: '\epsilon';            Hint: 'Epsilon'),
    (Command: '\zeta';               Hint: 'Zeta'),
    (Command: '\eta';                Hint: 'Eta'),
    (Command: '\theta';              Hint: 'Theta'),
    (Command: '\vartheta';           Hint: 'Theta Variant'),
    (Command: '\iota';               Hint: 'Iota'),
    (Command: '\kappa';              Hint: 'Kappa'),
    (Command: '\lambda';             Hint: 'Lambda'),
    (Command: '\mu';                 Hint: 'Mu'),
    (Command: '\nu';                 Hint: 'Nu'),
    (Command: '\xi';                 Hint: 'Xi'),
    (Command: '\omicron';            Hint: 'Omicron'),
    (Command: '\pi';                 Hint: 'Pi'),
    (Command: '\varpi';              Hint: 'Pi Variant'),
    (Command: '\rho';                Hint: 'Rho'),
    (Command: '\varrho';             Hint: 'Rho Variant'),
    (Command: '\sigma';              Hint: 'Sigma'),
    (Command: '\varsigma';           Hint: 'Sigma Variant'),
    (Command: '\tau';                Hint: 'Tau'),
    (Command: '\upsilon';            Hint: 'Upsilon'),
    (Command: '\varphi';             Hint: 'Phi Variant'),
    (Command: '\phi';                Hint: 'Phi'),
    (Command: '\chi';                Hint: 'Chi'),
    (Command: '\psi';                Hint: 'Psi'),
    (Command: '\omega';              Hint: 'Omega'),
    (Command: '\Alpha';              Hint: 'Alpha'),
    (Command: '\Beta';               Hint: 'Beta'),
    (Command: '\Gamma';              Hint: 'Gamma'),
    (Command: '\Delta';              Hint: 'Delta'),
    (Command: '\Epsilon';            Hint: 'Epsilon'),
    (Command: '\Eta';                Hint: 'Eta'),
    (Command: '\Zeta';               Hint: 'Zeta'),
    (Command: '\Theta';              Hint: 'Theta'),
    (Command: '\Iota';               Hint: 'Iota'),
    (Command: '\Kappa';              Hint: 'Kappa'),
    (Command: '\Lambda';             Hint: 'Lambda'),
    (Command: '\Mu';                 Hint: 'Mu'),
    (Command: '\Nu';                 Hint: 'Nu'),
    (Command: '\Xi';                 Hint: 'Xi'),
    (Command: '\Omicron';            Hint: 'Omicron'),
    (Command: '\Pi';                 Hint: 'Pi'),
    (Command: '\Rho';                Hint: 'Rho'),
    (Command: '\Sigma';              Hint: 'Sigma'),
    (Command: '\Tau';                Hint: 'Tau'),
    (Command: '\Upsilon';            Hint: 'Upsilon'),
    (Command: '\Phi';                Hint: 'Phi'),
    (Command: '\Chi';                Hint: 'Chi'),
    (Command: '\Psi';                Hint: 'Psi'),
    (Command: '\Omega';              Hint: 'Omega'),
    (Command: '\mathcal{A}';         Hint: 'Script Capital A'),
    (Command: '\mathcal{B}';         Hint: 'Script Capital B'),
    (Command: '\mathcal{C}';         Hint: 'Script Capital C'),
    (Command: '\mathcal{D}';         Hint: 'Script Capital D'),
    (Command: '\mathcal{E}';         Hint: 'Script Capital E'),
    (Command: '\mathcal{F}';         Hint: 'Script Capital F'),
    (Command: '\mathcal{G}';         Hint: 'Script Capital G'),
    (Command: '\mathcal{H}';         Hint: 'Script Capital H'),
    (Command: '\mathcal{I}';         Hint: 'Script Capital I'),
    (Command: '\mathcal{J}';         Hint: 'Script Capital J'),
    (Command: '\mathcal{K}';         Hint: 'Script Capital K'),
    (Command: '\mathcal{L}';         Hint: 'Script Capital L'),
    (Command: '\mathcal{M}';         Hint: 'Script Capital M'),
    (Command: '\mathcal{N}';         Hint: 'Script Capital N'),
    (Command: '\mathcal{O}';         Hint: 'Script Capital O'),
    (Command: '\mathcal{P}';         Hint: 'Script Capital P'),
    (Command: '\mathcal{Q}';         Hint: 'Script Capital Q'),
    (Command: '\mathcal{R}';         Hint: 'Script Capital R'),
    (Command: '\mathcal{S}';         Hint: 'Script Capital S'),
    (Command: '\mathcal{T}';         Hint: 'Script Capital T'),
    (Command: '\mathcal{U}';         Hint: 'Script Capital U'),
    (Command: '\mathcal{V}';         Hint: 'Script Capital V'),
    (Command: '\mathcal{W}';         Hint: 'Script Capital W'),
    (Command: '\mathcal{X}';         Hint: 'Script Capital X'),
    (Command: '\mathcal{Y}';         Hint: 'Script Capital Y'),
    (Command: '\mathcal{Z}';         Hint: 'Script Capital Z'),
    (Command: '\mathcal{a}';         Hint: 'Script Lowercase A'),
    (Command: '\mathcal{b}';         Hint: 'Script Lowercase B'),
    (Command: '\mathcal{c}';         Hint: 'Script Lowercase C'),
    (Command: '\mathcal{d}';         Hint: 'Script Lowercase D'),
    (Command: '\mathcal{e}';         Hint: 'Script Lowercase E'),
    (Command: '\mathcal{f}';         Hint: 'Script Lowercase F'),
    (Command: '\mathcal{g}';         Hint: 'Script Lowercase G'),
    (Command: '\mathcal{h}';         Hint: 'Script Lowercase H'),
    (Command: '\mathcal{i}';         Hint: 'Script Lowercase I'),
    (Command: '\mathcal{j}';         Hint: 'Script Lowercase J'),
    (Command: '\mathcal{k}';         Hint: 'Script Lowercase K'),
    (Command: '\mathcal{l}';         Hint: 'Script Lowercase L'),
    (Command: '\mathcal{m}';         Hint: 'Script Lowercase M'),
    (Command: '\mathcal{n}';         Hint: 'Script Lowercase N'),
    (Command: '\mathcal{o}';         Hint: 'Script Lowercase O'),
    (Command: '\mathcal{p}';         Hint: 'Script Lowercase P'),
    (Command: '\mathcal{q}';         Hint: 'Script Lowercase Q'),
    (Command: '\mathcal{r}';         Hint: 'Script Lowercase R'),
    (Command: '\mathcal{s}';         Hint: 'Script Lowercase S'),
    (Command: '\mathcal{t}';         Hint: 'Script Lowercase T'),
    (Command: '\mathcal{u}';         Hint: 'Script Lowercase U'),
    (Command: '\mathcal{v}';         Hint: 'Script Lowercase V'),
    (Command: '\mathcal{w}';         Hint: 'Script Lowercase W'),
    (Command: '\mathcal{x}';         Hint: 'Script Lowercase X'),
    (Command: '\mathcal{y}';         Hint: 'Script Lowercase Y'),
    (Command: '\mathcal{z}';         Hint: 'Script Lowercase Z'),
    (Command: '\mathfrak{A}';        Hint: 'Fraktur Capital A'),
    (Command: '\mathfrak{B}';        Hint: 'Fraktur Capital B'),
    (Command: '\mathfrak{C}';        Hint: 'Fraktur Capital C'),
    (Command: '\mathfrak{D}';        Hint: 'Fraktur Capital D'),
    (Command: '\mathfrak{E}';        Hint: 'Fraktur Capital E'),
    (Command: '\mathfrak{F}';        Hint: 'Fraktur Capital F'),
    (Command: '\mathfrak{G}';        Hint: 'Fraktur Capital G'),
    (Command: '\mathfrak{H}';        Hint: 'Fraktur Capital H'),
    (Command: '\mathfrak{I}';        Hint: 'Fraktur Capital I'),
    (Command: '\mathfrak{J}';        Hint: 'Fraktur Capital J'),
    (Command: '\mathfrak{K}';        Hint: 'Fraktur Capital K'),
    (Command: '\mathfrak{L}';        Hint: 'Fraktur Capital L'),
    (Command: '\mathfrak{M}';        Hint: 'Fraktur Capital M'),
    (Command: '\mathfrak{N}';        Hint: 'Fraktur Capital N'),
    (Command: '\mathfrak{O}';        Hint: 'Fraktur Capital O'),
    (Command: '\mathfrak{P}';        Hint: 'Fraktur Capital P'),
    (Command: '\mathfrak{Q}';        Hint: 'Fraktur Capital Q'),
    (Command: '\mathfrak{R}';        Hint: 'Fraktur Capital R'),
    (Command: '\mathfrak{S}';        Hint: 'Fraktur Capital S'),
    (Command: '\mathfrak{T}';        Hint: 'Fraktur Capital T'),
    (Command: '\mathfrak{U}';        Hint: 'Fraktur Capital U'),
    (Command: '\mathfrak{V}';        Hint: 'Fraktur Capital V'),
    (Command: '\mathfrak{W}';        Hint: 'Fraktur Capital W'),
    (Command: '\mathfrak{X}';        Hint: 'Fraktur Capital X'),
    (Command: '\mathfrak{Y}';        Hint: 'Fraktur Capital Y'),
    (Command: '\mathfrak{Z}';        Hint: 'Fraktur Capital Z'),
    (Command: '\mathfrak{a}';        Hint: 'Fraktur Lowercase A'),
    (Command: '\mathfrak{b}';        Hint: 'Fraktur Lowercase B'),
    (Command: '\mathfrak{c}';        Hint: 'Fraktur Lowercase C'),
    (Command: '\mathfrak{d}';        Hint: 'Fraktur Lowercase D'),
    (Command: '\mathfrak{e}';        Hint: 'Fraktur Lowercase E'),
    (Command: '\mathfrak{f}';        Hint: 'Fraktur Lowercase F'),
    (Command: '\mathfrak{g}';        Hint: 'Fraktur Lowercase G'),
    (Command: '\mathfrak{h}';        Hint: 'Fraktur Lowercase H'),
    (Command: '\mathfrak{i}';        Hint: 'Fraktur Lowercase I'),
    (Command: '\mathfrak{j}';        Hint: 'Fraktur Lowercase J'),
    (Command: '\mathfrak{k}';        Hint: 'Fraktur Lowercase K'),
    (Command: '\mathfrak{l}';        Hint: 'Fraktur Lowercase L'),
    (Command: '\mathfrak{m}';        Hint: 'Fraktur Lowercase M'),
    (Command: '\mathfrak{n}';        Hint: 'Fraktur Lowercase N'),
    (Command: '\mathfrak{o}';        Hint: 'Fraktur Lowercase O'),
    (Command: '\mathfrak{p}';        Hint: 'Fraktur Lowercase P'),
    (Command: '\mathfrak{q}';        Hint: 'Fraktur Lowercase Q'),
    (Command: '\mathfrak{r}';        Hint: 'Fraktur Lowercase R'),
    (Command: '\mathfrak{s}';        Hint: 'Fraktur Lowercase S'),
    (Command: '\mathfrak{t}';        Hint: 'Fraktur Lowercase T'),
    (Command: '\mathfrak{u}';        Hint: 'Fraktur Lowercase U'),
    (Command: '\mathfrak{v}';        Hint: 'Fraktur Lowercase V'),
    (Command: '\mathfrak{w}';        Hint: 'Fraktur Lowercase W'),
    (Command: '\mathfrak{x}';        Hint: 'Fraktur Lowercase X'),
    (Command: '\mathfrak{y}';        Hint: 'Fraktur Lowercase Y'),
    (Command: '\mathfrak{z}';        Hint: 'Fraktur Lowercase Z'),
    (Command: '\mathbb{A}';          Hint: 'Double-Struck Capital A'),
    (Command: '\mathbb{B}';          Hint: 'Double-Struck Capital B'),
    (Command: '\mathbb{C}';          Hint: 'Double-Struck Capital C'),
    (Command: '\mathbb{D}';          Hint: 'Double-Struck Capital D'),
    (Command: '\mathbb{E}';          Hint: 'Double-Struck Capital E'),
    (Command: '\mathbb{F}';          Hint: 'Double-Struck Capital F'),
    (Command: '\mathbb{G}';          Hint: 'Double-Struck Capital G'),
    (Command: '\mathbb{H}';          Hint: 'Double-Struck Capital H'),
    (Command: '\mathbb{I}';          Hint: 'Double-Struck Capital I'),
    (Command: '\mathbb{J}';          Hint: 'Double-Struck Capital J'),
    (Command: '\mathbb{K}';          Hint: 'Double-Struck Capital K'),
    (Command: '\mathbb{L}';          Hint: 'Double-Struck Capital L'),
    (Command: '\mathbb{M}';          Hint: 'Double-Struck Capital M'),
    (Command: '\mathbb{N}';          Hint: 'Double-Struck Capital N'),
    (Command: '\mathbb{O}';          Hint: 'Double-Struck Capital O'),
    (Command: '\mathbb{P}';          Hint: 'Double-Struck Capital P'),
    (Command: '\mathbb{Q}';          Hint: 'Double-Struck Capital Q'),
    (Command: '\mathbb{R}';          Hint: 'Double-Struck Capital R'),
    (Command: '\mathbb{S}';          Hint: 'Double-Struck Capital S'),
    (Command: '\mathbb{T}';          Hint: 'Double-Struck Capital T'),
    (Command: '\mathbb{U}';          Hint: 'Double-Struck Capital U'),
    (Command: '\mathbb{V}';          Hint: 'Double-Struck Capital V'),
    (Command: '\mathbb{W}';          Hint: 'Double-Struck Capital W'),
    (Command: '\mathbb{X}';          Hint: 'Double-Struck Capital X'),
    (Command: '\mathbb{Y}';          Hint: 'Double-Struck Capital Y'),
    (Command: '\mathbb{Z}';          Hint: 'Double-Struck Capital Z'),
    (Command: '\mathbb{a}';          Hint: 'Double-Struck Lowercase A'),
    (Command: '\mathbb{b}';          Hint: 'Double-Struck Lowercase B'),
    (Command: '\mathbb{c}';          Hint: 'Double-Struck Lowercase C'),
    (Command: '\mathbb{d}';          Hint: 'Double-Struck Lowercase D'),
    (Command: '\mathbb{e}';          Hint: 'Double-Struck Lowercase E'),
    (Command: '\mathbb{f}';          Hint: 'Double-Struck Lowercase F'),
    (Command: '\mathbb{g}';          Hint: 'Double-Struck Lowercase G'),
    (Command: '\mathbb{h}';          Hint: 'Double-Struck Lowercase H'),
    (Command: '\mathbb{i}';          Hint: 'Double-Struck Lowercase I'),
    (Command: '\mathbb{j}';          Hint: 'Double-Struck Lowercase J'),
    (Command: '\mathbb{k}';          Hint: 'Double-Struck Lowercase K'),
    (Command: '\mathbb{l}';          Hint: 'Double-Struck Lowercase L'),
    (Command: '\mathbb{m}';          Hint: 'Double-Struck Lowercase M'),
    (Command: '\mathbb{n}';          Hint: 'Double-Struck Lowercase N'),
    (Command: '\mathbb{o}';          Hint: 'Double-Struck Lowercase O'),
    (Command: '\mathbb{p}';          Hint: 'Double-Struck Lowercase P'),
    (Command: '\mathbb{q}';          Hint: 'Double-Struck Lowercase Q'),
    (Command: '\mathbb{r}';          Hint: 'Double-Struck Lowercase R'),
    (Command: '\mathbb{s}';          Hint: 'Double-Struck Lowercase S'),
    (Command: '\mathbb{t}';          Hint: 'Double-Struck Lowercase T'),
    (Command: '\mathbb{u}';          Hint: 'Double-Struck Lowercase U'),
    (Command: '\mathbb{v}';          Hint: 'Double-Struck Lowercase V'),
    (Command: '\mathbb{w}';          Hint: 'Double-Struck Lowercase W'),
    (Command: '\mathbb{x}';          Hint: 'Double-Struck Lowercase X'),
    (Command: '\mathbb{y}';          Hint: 'Double-Struck Lowercase Y'),
    (Command: '\mathbb{z}';          Hint: 'Double-Struck Lowercase Z'),
    (Command: '\eth';                Hint: 'Latin Small Letter Eth'),
    (Command: '\Euler';              Hint: 'Euler Constant'),
    (Command: '\Digamma';            Hint: 'Greek Letter Digamma'),
    (Command: '\Finv';               Hint: 'Turned Capital F'),
    (Command: '\Planckconst';        Hint: 'Plank Constant'),
    (Command: '\hslash';             Hint: 'Plank Constant Over Two Pi'),
    (Command: '\turnediota';         Hint: 'Turned Greek Small Letter Iota'),
    (Command: '\imath';              Hint: 'Latin Small Letter Dotless I'),
    (Command: '\Im';                 Hint: 'Imaginary Part'),
    (Command: '\jmath';              Hint: 'Latin Small Letter Dotless J'),
    (Command: '\varkappa';           Hint: 'Greek Kappa Symbol'),
    (Command: '\ell';                Hint: 'Script Lowercase L'),
    (Command: '\wp';                 Hint: 'Weierstrass Elliptic Function'),
    (Command: '\Re';                 Hint: 'Real Part'),
    (Command: '\mho';                Hint: 'Inverted Ohm Sign'),
    (Command: '\Angstrom';           Hint: 'Angstrom Sign'),
    (Command: '\estimated';          Hint: 'Estimated Symbol'),
    (Command: '\gimel';              Hint: 'Gimel Symbol (Third Transfinite Cardinal)'),
    (Command: '\daleth';             Hint: 'Dalet Symbol (Fourth Transfinite Cardinal)'),
    (Command: '\fracslash';          Hint: 'Division Slash'),
    (Command: '\bigoplus';           Hint: 'N-ary Circled Plus Operator'),
    (Command: '\circ';               Hint: 'Composite Function'),
    (Command: '\biguplus';           Hint: 'N-ary Union Operator with Plus'),
    (Command: '\cdot';               Hint: 'Dot Operator'),
    (Command: '\uplus';              Hint: 'Multiset Union'),
    (Command: '\sqcap';              Hint: 'Square Cap'),
    (Command: '\sqcup';              Hint: 'Square Cup'),
    (Command: '\wedge';              Hint: 'Logical And'),
    (Command: '\vee';                Hint: 'Logical Or'),
    (Command: '\nless';              Hint: 'Not Less Than'),
    (Command: '\nleq';               Hint: 'Neither Less Than nor Equal To'),
    (Command: '\ngtr';               Hint: 'Not Greater Than'),
    (Command: '\ngeq';               Hint: 'Neither Greater Than nor Equal To'),
    (Command: '\simeq';              Hint: 'Asymptotically Equal To'),
    (Command: '\bigudot';            Hint: 'N-ary Union Operator with Dot'),
    (Command: '\nequiv';             Hint: 'Not Identical To'),
    (Command: '\nsimeq';             Hint: 'Not Asymptotically Equal To'),
    (Command: '\napprox';            Hint: 'Not Almost Equal To'),
    (Command: '\ncong';              Hint: 'Neither Asymptotically nor Almost Equal To'),
    (Command: '\notin';              Hint: 'Not an Element Of'),
    (Command: '\subset';             Hint: 'Subset Of'),
    (Command: '\supset';             Hint: 'Superset Of'),
    (Command: '\subseteq';           Hint: 'Subset Of or Equal To'),
    (Command: '\supseteq';           Hint: 'Superset Of or Equal To'),
    (Command: '\prec';               Hint: 'Precedes'),
    (Command: '\succ';               Hint: 'Succeeds'),
    (Command: '\preceq';             Hint: 'Precedes or Equal To'),
    (Command: '\succeq';             Hint: 'Succeeds or Equal To'),
    (Command: '\sqsubset';           Hint: 'Square Image Of'),
    (Command: '\sqsupset';           Hint: 'Square Original Of'),
    (Command: '\sqsubseteq';         Hint: 'Square Image Of or Equal To'),
    (Command: '\sqsupseteq';         Hint: 'Square Original Of or Equal To'),
    (Command: '\parallel';           Hint: 'Parallel To'),
    (Command: '\bot';                Hint: 'Perpendicular To (Orthogonal To)'),
    (Command: '\vdash';              Hint: 'Proves (Implies, Yields)'),
    (Command: '\dashv';              Hint: 'Does not Yields'),
    (Command: '\bowtie';             Hint: 'Join'),
    (Command: '\asymp';              Hint: 'Equivalent To'),
    (Command: '\sum';                Hint: 'Summation Sign'),
    (Command: '\int';                Hint: 'Integral'),
    (Command: '\iint';               Hint: 'Double Integral'),
    (Command: '\iiint';              Hint: 'Triple Integral'),
    (Command: '\oint';               Hint: 'Contour Integral'),
    (Command: '\oiint';              Hint: 'Surface Integral'),
    (Command: '\oiiint';             Hint: 'Volume Integral'),
    (Command: '\varointclockwise';   Hint: 'Clockwise Contour Integral'),
    (Command: '\ointctrclockwise';   Hint: 'Counterclockwise Contour Integral'),
    (Command: '\prod';               Hint: 'Product Sign'),
    (Command: '\coprod';             Hint: 'Coproduct Sign'),
    (Command: '\bigcap';             Hint: 'N-ary Intersection'),
    (Command: '\bigcup';             Hint: 'N-ary Union'),
    (Command: '\bigwedge';           Hint: 'N-ary Logical And'),
    (Command: '\bigvee';             Hint: 'N-ary Logical Or'),
    (Command: '\bigodot';            Hint: 'N-ary Circled Dot Operator'),
    (Command: '\bigotimes';          Hint: 'N-ary Circled Times Operator'),
    (Command: '\increment';          Hint: 'Increment'),
    (Command: '\dotplus';            Hint: 'Dot Plus'),
    (Command: '\dotminus';           Hint: 'Dot Minus (Symmetric Difference'),
    (Command: '\setminus';           Hint: 'Set Minus'),
    (Command: '\Cap';                Hint: 'Double Intersection'),
    (Command: '\Cup';                Hint: 'Double Union'),
    (Command: '\boxminus';           Hint: 'Squared Minus'),
    (Command: '\boxtimes';           Hint: 'Squared Times'),
    (Command: '\boxdot';             Hint: 'Squared Dot Operator'),
    (Command: '\boxplus';            Hint: 'Squared Plus'),
    (Command: '\divideontimes';      Hint: 'Division Times'),
    (Command: '\ltimes';             Hint: 'Left Normal Factor Semidirect Product'),
    (Command: '\rtimes';             Hint: 'Right Normal Factor Semidirect Product'),
    (Command: '\leftthreetimes';     Hint: 'Left Semidirect Product'),
    (Command: '\rightthreetimes';    Hint: 'Right Semidirect Product'),
    (Command: '\curlyvee';           Hint: 'Curly Logical And'),
    (Command: '\curlywedge';         Hint: 'Curly Logical Or'),
    (Command: '\circleddash';        Hint: 'Circled Dash'),
    (Command: '\intercal';           Hint: 'Intercalate'),
    (Command: '\oplus';              Hint: 'Circled Plus'),
    (Command: '\ominus';             Hint: 'Circled Minus'),
    (Command: '\otimes';             Hint: 'Circled Times'),
    (Command: '\oslash';             Hint: 'Circled Division Slash'),
    (Command: '\odot';               Hint: 'Circled Dot Operator'),
    (Command: '\circledast';         Hint: 'Circled Asterisk Operator'),
    (Command: '\circledcirc';        Hint: 'Circled Ring Operator'),
    (Command: '\dagger';             Hint: 'Dagger'),
    (Command: '\ddagger';            Hint: 'Double Dagger'),
    (Command: '\star';               Hint: 'Star Operator'),
    (Command: '\diamond';            Hint: 'Diamond Operator'),
    (Command: '\wr';                 Hint: 'Wreath Product'),
    (Command: '\bigtriangleup';      Hint: 'White Up-Pointing Triangle'),
    (Command: '\bigsqcap';           Hint: 'N-ary Square Intersection Operator'),
    (Command: '\bigsqcup';           Hint: 'N-ary Square Union Operator'),
    (Command: '\because';            Hint: 'Because'),
    (Command: '\lll';                Hint: 'Very Much Less Than'),
    (Command: '\ggg';                Hint: 'Very Much Greater Than'),
    (Command: '\leqq';               Hint: 'Less Than Over Equal To'),
    (Command: '\geqq';               Hint: 'Greater Than Over Equal To'),
    (Command: '\lesssim';            Hint: 'Less Than or Equivalent To'),
    (Command: '\gtrsim';             Hint: 'Greater Than or Equivalent To'),
    (Command: '\lessdot';            Hint: 'Less Than with Dot'),
    (Command: '\gtrdot';             Hint: 'Greater Than with Dot'),
    (Command: '\lessgtr';            Hint: 'Less Than or Greater Than'),
    (Command: '\lesseqgtr';          Hint: 'Less Than, Equal to or Greater Than'),
    (Command: '\gtrless';            Hint: 'Greater Than or Less Than'),
    (Command: '\gtreqless';          Hint: 'Greater Than, Equal to or Less Than'),
    (Command: '\Doteq';              Hint: 'Geometrically Equal To'),
    (Command: '\fallingdotseq';      Hint: 'Approximately Equal to or The Image Of'),
    (Command: '\risingdotseq';       Hint: 'Image Of or Approximately Equal To'),
    (Command: '\approxeq';           Hint: 'Almost Equal or Equal To'),
    (Command: '\backsimeq';          Hint: 'Reversed Tilde Equals'),
    (Command: '\preccurlyeq';        Hint: 'Equal to or Precedes'),
    (Command: '\succcurlyeq';        Hint: 'Equal to or Succeeds'),
    (Command: '\curlyeqprec';        Hint: 'Precedes or Equivalent To'),
    (Command: '\curlyeqsucc';        Hint: 'Succeeds or Equivalent To'),
    (Command: '\precsim';            Hint: 'Precedes or Equivalent To'),
    (Command: '\succsim';            Hint: 'Succeeds or Equivalent To'),
    (Command: '\eqless';             Hint: 'Equal to or Less Than'),
    (Command: '\eqgtr';              Hint: 'Equal to or Greater Than'),
    (Command: '\vartriangleleft';    Hint: 'Normal Subgroup Of'),
    (Command: '\vartriangleright';   Hint: 'Contains as Normal Subgroup'),
    (Command: '\trianglelefteq';     Hint: 'Normal Subgroup of or Equal To'),
    (Command: '\trianglerighteq';    Hint: 'Contains as Normal Subgroup or Equal To'),
    (Command: '\vDash';              Hint: 'True'),
    (Command: '\Subset';             Hint: 'Double Subset'),
    (Command: '\Supset';             Hint: 'Double Superset'),
    (Command: '\Vdash';              Hint: 'Forces'),
    (Command: '\Vvdash';             Hint: 'Triple Vertical Bar Right Turnstile'),
    (Command: '\eqcirc';             Hint: 'Ring in Equal To'),
    (Command: '\circeq';             Hint: 'Ring Equal To'),
    (Command: '\triangleq';          Hint: 'Delta Equal To'),
    (Command: '\bumpeq';             Hint: 'Difference Between'),
    (Command: '\Bumpeq';             Hint: 'Geometrically Equivalent To'),
    (Command: '\between';            Hint: 'Between'),
    (Command: '\pitchfork';          Hint: 'Pitchfork'),
    (Command: '\doteq';              Hint: 'Approaches The Limit'),
    (Command: '\updownarrow';        Hint: 'Up-Down Arrow'),
    (Command: '\Leftarrow';          Hint: 'Left Double Arrow'),
    (Command: '\Rightarrow';         Hint: 'Right Double Arrow'),
    (Command: '\Uparrow';            Hint: 'Up Double Arrow'),
    (Command: '\Downarrow';          Hint: 'Down Double Arrow'),
    (Command: '\Leftrightarrow';     Hint: 'Left-Right Double Arrow'),
    (Command: '\Updownarrow';        Hint: 'Up-Down Double Arrow'),
    (Command: '\longleftarrow';      Hint: 'Long Left Arrow'),
    (Command: '\longrightarrow';     Hint: 'Long Right Arrow'),
    (Command: '\longleftrightarrow'; Hint: 'Long Left-Right Arrow'),
    (Command: '\Longleftarrow';      Hint: 'Long Left Double Arrow'),
    (Command: '\Longrightarrow';     Hint: 'Long Right Double Arrow'),
    (Command: '\Longleftrightarrow'; Hint: 'Long Left-Right Double Arrow'),
    (Command: '\nearrow';            Hint: 'Diagonal Upwards Right Arrow'),
    (Command: '\nwarrow';            Hint: 'Diagonal Upwards Left Arrow'),
    (Command: '\searrow';            Hint: 'Diagonal Downwards Right Arrow'),
    (Command: '\swarrow';            Hint: 'Diagonal Downwards Left Arrow'),
    (Command: '\nleftarrow';         Hint: 'Left Arrow with Stroke'),
    (Command: '\nrightarrow';        Hint: 'Right Arrow with Stroke'),
    (Command: '\nleftrightarrow';    Hint: 'Left-Right Arrow with Stroke'),
    (Command: '\nLeftarrow';         Hint: 'Left Double Arrow with Stroke'),
    (Command: '\nRightarrow';        Hint: 'Right Double Arrow with Stroke'),
    (Command: '\nLeftrightarrow';    Hint: 'Left-Right Double Arrow with Stroke'),
    (Command: '\leftdasharrow';      Hint: 'Left Dashed Arrow'),
    (Command: '\rightdasharrow';     Hint: 'Right Dashed Arrow'),
    (Command: '\mapsfrom';           Hint: 'Left Arrow From Bar'),
    (Command: '\mapsto';             Hint: 'Right Arrow From Bar'),
    (Command: '\longmapsfrom';       Hint: 'Long Left Arrow From Bar'),
    (Command: '\longmapsto';         Hint: 'Long Right Arrow From Bar'),
    (Command: '\hookleftarrow';      Hint: 'Left Arrow with Hook'),
    (Command: '\hookrightarrow';     Hint: 'Right Arrow with Hook'),
    (Command: '\leftharpoonup';      Hint: 'Left Harpoon with Barb Up'),
    (Command: '\leftharpoondown';    Hint: 'Left Harpoon with Barb Down'),
    (Command: '\rightharpoonup';     Hint: 'Right Harpoon with Barb Up'),
    (Command: '\rightharpoondown';   Hint: 'Right Harpoon with Barb Down'),
    (Command: '\upharpoonleft';      Hint: 'Up Harpoon with Barb Left'),
    (Command: '\upharpoonright';     Hint: 'Up Harpoon with Barb Right'),
    (Command: '\downharpoonleft';    Hint: 'Down Harpoon with Barb Left'),
    (Command: '\downharpoonright';   Hint: 'Down Harpoon with Barb Right'),
    (Command: '\leftrightharpoons';  Hint: 'Left Harpoon Over Right Harpoon'),
    (Command: '\rightleftharpoons';  Hint: 'Right Harpoon Over Left Harpoon'),
    (Command: '\leftleftarrows';     Hint: 'Left Paired Arrows'),
    (Command: '\rightrightarrows';   Hint: 'Right Paired Arrows'),
    (Command: '\upuparrows';         Hint: 'Up Paired Arrows'),
    (Command: '\downdownarrows';     Hint: 'Down Paired Arrows'),
    (Command: '\leftrightarrows';    Hint: 'Left Arrow Over Right Arrow'),
    (Command: '\rightleftarrows';    Hint: 'Right Arrow Over Left Arrow'),
    (Command: '\looparrowleft';      Hint: 'Left Arrow with Loop'),
    (Command: '\looparrowright';     Hint: 'Right Arrow with Loop'),
    (Command: '\leftarrowtail';      Hint: 'Left Arrow with Tail'),
    (Command: '\rightarrowtail';     Hint: 'Right Arrow with Tail'),
    (Command: '\Lsh';                Hint: 'Up Arrow with Tip Left'),
    (Command: '\Rsh';                Hint: 'Up Arrow with Tip Right'),
    (Command: '\dlsh';               Hint: 'Down Arrow with Tip Left'),
    (Command: '\drsh';               Hint: 'Down Arrow with Tip Right'),
    (Command: '\Lleftarrow';         Hint: 'Left Triple Arrow'),
    (Command: '\Rrightarrow';        Hint: 'Right Triple Arrow'),
    (Command: '\twoheadleftarrow';   Hint: 'Left Two Headed Arrow'),
    (Command: '\twoheadrightarrow';  Hint: 'Right Two Headed Arrow'),
    (Command: '\curvearrowleft';     Hint: 'Counterclockwise Top Semicircle Arrow'),
    (Command: '\curvearrowright';    Hint: 'Clockwise Top Semicircle Arrow'),
    (Command: '\circlearrowleft';    Hint: 'Counterclockwise Open Semicircle Arrow'),
    (Command: '\circlearrowright';   Hint: 'Clockwise Open Semicircle Arrow'),
    (Command: '\multimap';           Hint: 'Multimap'),
    (Command: '\leftrightsquigarrow';Hint: 'Left-Right Wave Arrow'),
    (Command: '\leftwavearrow';      Hint: 'Left Wave Arrow'),
    (Command: '\rightwavearrow';     Hint: 'Right Wave Arrow'),
    (Command: '\leftsquigarrow';     Hint: 'Left Squiggle Arrow'),
    (Command: '\rightsquigarrow';    Hint: 'Right Squiggle Arrow'),
    (Command: '\nsim';               Hint: 'Not Tilde'),
    (Command: '\nasymp';             Hint: 'Not Equivalent To'),
    (Command: '\lneqq';              Hint: 'Less Than but Not Equal To'),
    (Command: '\gneqq';              Hint: 'Greater Than but Not Equal To'),
    (Command: '\nprec';              Hint: 'Does Not Precede'),
    (Command: '\nsucc';              Hint: 'Does Not Succeed'),
    (Command: '\npreceq';            Hint: 'Does Not Precede or Equal'),
    (Command: '\nsucceq';            Hint: 'Does Not Succeed or Equal'),
    (Command: '\nni';                Hint: 'Does Not Contain as Member'),
    (Command: '\nsubset';            Hint: 'Not a Subset Of'),
    (Command: '\nsupset';            Hint: 'Not a Superset Of'),
    (Command: '\nsubseteq';          Hint: 'Neither a Subset of nor Equal To'),
    (Command: '\nsupseteq';          Hint: 'Neither a Superset of nor Equal To'),
    (Command: '\subsetneq';          Hint: 'Subset Of with Not Equal To'),
    (Command: '\supsetneq';          Hint: 'Superset Of with Not Equal To'),
    (Command: '\nsqsubseteq';        Hint: 'Not Square Image Of or Equal To'),
    (Command: '\nsqsupseteq';        Hint: 'Not Square Original Of or Equal To'),
    (Command: '\lnsim';              Hint: 'Less Than but Not Equivalent To'),
    (Command: '\gnsim';              Hint: 'Greater Than but Not Equivalent To'),
    (Command: '\precnsim';           Hint: 'Precedes Than but Not Equivalent To'),
    (Command: '\succnsim';           Hint: 'Succeeds Than but Not Equivalent To'),
    (Command: '\ntriangleleft';      Hint: 'Not Normal Subgroup Of'),
    (Command: '\ntriangleright';     Hint: 'Does Not Contain as Normal Subgroup Of'),
    (Command: '\ntrianglelefteq';    Hint: 'Not Normal Subgroup Of or Equal To'),
    (Command: '\ntrianglerighteq';   Hint: 'Does Not Contain as Normal Subgroup Of or Equal To'),
    (Command: '\nmid';               Hint: 'Does Not Divide'),
    (Command: '\nparallel';          Hint: 'Does Not Parallel'),
    (Command: '\nvdash';             Hint: 'Does Not Prove'),
    (Command: '\nvDash';             Hint: 'Not True'),
    (Command: '\nVdash';             Hint: 'Does Not Force'),
    (Command: '\nVDash';             Hint: 'Negated Double Vertical Bar Double Right Turnstile'),
    (Command: '\rightangle';         Hint: 'Right Angle'),
    (Command: '\angle';              Hint: 'Angle'),
    (Command: '\measuredangle';      Hint: 'Measured Angle'),
    (Command: '\sphericalangle';     Hint: 'Spherical Angle'),
    (Command: '\measuredrightangle'; Hint: 'Right Angle with Arc'),
    (Command: '\varlrtriangle';      Hint: 'Right Triangle'),
    (Command: '\hash';               Hint: 'Equal and Parallel To'),
    (Command: '\mathratio';          Hint: 'Ratio'),
    (Command: '\Proportion';         Hint: 'Proportional'),
    (Command: '\intclockwise';       Hint: 'Clockwise Integral'),
    (Command: '\backsim';            Hint: 'Reversed Tilde')
  );


const
  MathRanges: array[TMathSymbolType] of TMathSymbolRanges = (
    (SymbType: mstBasic;
     ImageIndexes: (1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17,
                    18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32,
                    341, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46,
                    56, 57, 58, 59, 60, 61, 64, 65, 69, 73, 75, 77, 79, 81, 85,
                    47, 48, 49, 50, 51, 52, 53, 54, 55,
                    -1, -1, -1)),

    (SymbType: mstGreekLower;
     ImageIndexes: (56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70,
                    71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1)),

    (SymbType: mstGreekUpper;
     ImageIndexes: (86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99, 100,
                    101, 102, 103, 104, 105, 106, 107, 108, 109,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
                    -1, -1, -1, -1, -1
                    )),

    (SymbType: mstLetterLike;
     ImageIndexes: (20, 21, 216, 22, 266, 267, 268, 269, 142, 117, 169, 270, 271,
                    272, 273, 274, 275, 276, 121, 277, 227, 278, 230, 127, 279,
                    231, 239, 280, 281, 111, 282, 140, 34, 35, 115, 122, 150, 53,
                    54, 283, 284,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
                    -1, -1, -1
                    )),

    (SymbType: mstBinaryOperators;
     ImageIndexes: (44, 45, 7, 6, 1, 16, 9, 285, 47, 287, 48, 289, 27, 26, 290,
                    291, 292, 293, 294,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1
                    )),

    (SymbType: mstRelationOperators;
     ImageIndexes: (3, 4, 10, 12, 14, 15, 295, 296, 297, 298, 19, 5, 299, 18, 17,
                    301, 302, 303, 304, 9, 11, 13, 36, 37, 305, 306, 307, 308, 309,
                    310, 311, 312, 313, 314, 315, 316, 317, 318, 319, 320, 321, 322, 323,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
                    -1
                    )),

    (SymbType: mstNaryOperators;
     ImageIndexes: (324, 325, 326, 327, 328, 329, 330, 527, 331, 332, 333, 334, 335,
                    336, 337, 338, 339, 340, 286, 288, 300,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
                    -1, -1, -1, -1, -1, -1, -1, -1
                    )),

    (SymbType: mstAdvBinaryOperators;
     ImageIndexes: (342, 343, 344, 345, 346, 347, 348, 349, 350, 351, 352, 353,
                    354, 355, 356, 357, 358, 359, 360, 361, 362, 363, 364, 365,
                    366, 367, 368, 369, 370, 371, 372, 337, 338, 339, 340, 286,
                    373, 374, 288, 300,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
                    -1, -1, -1, -1
                    )),

    (SymbType: mstAdvRelationOperators;
     ImageIndexes: (43, 375, 376, 377, 378, 379, 380, 381, 382, 383, 384, 385,
                    386, 387, 388, 389, 390, 528, 391, 392, 393, 394, 395, 396,
                    397, 398, 399, 400, 308, 309, 401, 402, 403, 404, 405, 406,
                    407, 314, 315, 408, 409, 410, 411, 412, 413, 414, 9, 415,
                    416, 417, 322,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
                    -1, -1, -1, -1, -1, -1, -1, -1
                    )),

    (SymbType: mstArrows;
     ImageIndexes: (38, 40, 39, 41, 42, 418, 419, 420, 421, 422, 423, 424, 425,
                    426, 427, 428, 429, 430, 431, 432, 433, 434, 435, 436, 437,
                    438, 439, 440, 441, 442, 443, 444, 445, 446, 447, 448, 449,
                    450, 451, 452, 453, 454, 455, 456, 457, 458, 459, 460, 461,
                    462, 463, 464, 465, 466, 467, 468, 469, 470, 471, 472, 473,
                    474, 475, 476, 477, 478, 479, 480, 481, 482, 483, 484, 485, 486)),

    (SymbType: mstNegativeRelations;
     ImageIndexes: (4, 295, 297, 296, 298, 301, 487, 302, 303, 304, 488, 489,
                    490, 491, 492, 493, 494, 305, 495, 496, 497, 498, 499, 500,
                    501, 502, 503, 504, 505, 506, 507, 508, 509, 510, 511, 512,
                    513, 514, 515, 516, 517, 35,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
                    -1, -1
                    )),

    (SymbType: mstScripts;
     ImageIndexes: (110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121,
                    122, 123, 124, 125, 126, 127, 128, 129, 130, 131, 132, 133,
                    134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 144, 145,
                    146, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157,
                    158, 159, 160, 161,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
                    -1, -1, -1, -1, -1, -1, -1
                    )),

    (SymbType: mstFrankturs;
     ImageIndexes: (162, 163, 164, 165, 166, 167, 168, 169, 170, 171, 172, 173,
                    174, 175, 176, 177, 178, 179, 180, 181, 182, 183, 184, 185,
                    186, 187, 188, 189, 190, 191, 192, 193, 194, 195, 196, 197,
                    198, 199, 200, 201, 202, 203, 204, 205, 206, 207, 208, 209,
                    210, 211, 212, 213,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
                    -1, -1, -1, -1, -1, -1, -1
                    )),

    (SymbType: mstDoubleStruck;
     ImageIndexes: (214, 215, 216, 217, 218, 219, 220, 221, 222, 223, 224, 225,
                    226, 227, 228, 229, 230, 231, 232, 233, 234, 235, 236, 237,
                    238, 239, 240, 241, 242, 243, 244, 245, 246, 247, 248, 249,
                    250, 251, 252, 253, 254, 255, 256, 257, 258, 259, 260, 261,
                    262, 263, 264, 265,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
                    -1, -1, -1, -1, -1, -1, -1
                    )),

    (SymbType: mstGeometry;
     ImageIndexes: (518, 519, 520, 521, 522, 523, 524, 319, 512, 318, 513, 525,
                    526, 43, 375, 55,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
                    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1
                    ))
  );

  MathExpressions: array[0..222] of TMathExpressionRecord = (

  //Fractions
    (Size: meizNormal; ExprType: metFraction; Command: '\frac{%%}{%}'; Hint: 'Stacked Fraction'),
    (Size: meizNormal; ExprType: metFraction; Command: '\sfrac{%%}{%}'; Hint: 'Skewed Fraction'),
    (Size: meizNormal; ExprType: metFraction; Command: '\lfrac{%%}{%}'; Hint: 'Linear Fraction'),
    (Size: meizNormal; ExprType: metFraction; Command: '\tfrac{%%}{%}'; Hint: 'Small Fraction'),

  //Common Fractions
    (Size: meizNormal; ExprType: metCommonFraction; Command: '\frac{dy}{dx}'; Hint: 'Differential'),
    (Size: meizNormal; ExprType: metCommonFraction; Command: '\frac{\Delta y}{\Delta x}'; Hint: 'Differential'),
    (Size: meizNormal; ExprType: metCommonFraction; Command: '\frac{\partial y}{\partial x}'; Hint: 'Differential'),
    (Size: meizNormal; ExprType: metCommonFraction; Command: '\frac{\delta y}{\delta x}'; Hint: 'Differential'),
    (Size: meizNormal; ExprType: metCommonFraction; Command: '\frac{\pi}{2}'; Hint: 'Pi Over 2'),

  //Superscript and Subscript
    (Size: meizNormal; ExprType: metScript; Command: '%%^{%}'; Hint: 'Superscript'),
    (Size: meizNormal; ExprType: metScript; Command: '%%_{%}'; Hint: 'Subscript'),
    (Size: meizNormal; ExprType: metScript; Command: '%%_{%}^{%}'; Hint: 'Subscript-Superscript'),
    (Size: meizNormal; ExprType: metScript; Command: '{}_{%}^{%} %%'; Hint: 'Left Subscript-Superscript'),

  //Common Superscript and Subscript
    (Size: meizNormal; ExprType: metCommonScript; Command: 'x_{y^2}'; Hint: 'Script'),
    (Size: meizNormal; ExprType: metCommonScript; Command: 'e^{-i \omega t}'; Hint: 'Script'),
    (Size: meizNormal; ExprType: metCommonScript; Command: '\chi^2'; Hint: 'Script'),
    (Size: meizNormal; ExprType: metCommonScript; Command: '{}_n^1 Y'; Hint: 'Script'),

  //Radical
    (Size: meizNormal; ExprType: metRadical; Command: '\sqrt{%%}'; Hint: 'Square Root'),
    (Size: meizNormal; ExprType: metRadical; Command: '\sqrt[%]{%%}'; Hint: 'Radical With Degree'),
    (Size: meizNormal; ExprType: metRadical; Command: '\sqrt[2]{%%}'; Hint: 'Square Root With Degree'),
    (Size: meizNormal; ExprType: metRadical; Command: '\sqrt[3]{%%}'; Hint: 'Cubic Root'),

  //Common Radical
    (Size: meizNormalWide; ExprType: metCommonRadical; Command: '\frac{-b \pm \sqrt{b^2 - 4ac}}{2a}'; Hint: 'Radical'),
    (Size: meizNormalWide; ExprType: metCommonRadical; Command: '\sqrt{a^2 + b^2}'; Hint: 'Radical'),

  //Integrals
    (Size: meizNormal; ExprType: metIntegral; Command: '\int %%'; Hint: 'Integral'),
    (Size: meizNormal; ExprType: metIntegral; Command: '\int\nolimits_{%}^{%} %%'; Hint: 'Integral'),
    (Size: meizNormal; ExprType: metIntegral; Command: '\int\limits_{%}^{%} %%'; Hint: 'Integral'),
    (Size: meizNormal; ExprType: metIntegral; Command: '\iint %%'; Hint: 'Double Integral'),
    (Size: meizNormal; ExprType: metIntegral; Command: '\iint\nolimits_{%}^{%} %%'; Hint: 'Double Integral'),
    (Size: meizNormal; ExprType: metIntegral; Command: '\iint\limits_{%}^{%} %%'; Hint: 'Double Integral'),
    (Size: meizNormal; ExprType: metIntegral; Command: '\iiint %%'; Hint: 'Triple Integral'),
    (Size: meizNormal; ExprType: metIntegral; Command: '\iiint\nolimits_{%}^{%} %%'; Hint: 'Triple Integral'),
    (Size: meizNormal; ExprType: metIntegral; Command: '\iiint\limits_{%}^{%} %%'; Hint: 'Triple Integral'),

  //Contour Integral
    (Size: meizNormal; ExprType: metContourIntegral; Command: '\oint %%'; Hint: 'Contour Integral'),
    (Size: meizNormal; ExprType: metContourIntegral; Command: '\oint\nolimits_{%}^{%} %%'; Hint: 'Contour Integral'),
    (Size: meizNormal; ExprType: metContourIntegral; Command: '\oint\limits_{%}^{%} %%'; Hint: 'Contour Integral'),
    (Size: meizNormal; ExprType: metContourIntegral; Command: '\oiint %%'; Hint: 'Surface Integral'),
    (Size: meizNormal; ExprType: metContourIntegral; Command: '\oiint\nolimits_{%}^{%} %%'; Hint: 'Surface Integral'),
    (Size: meizNormal; ExprType: metContourIntegral; Command: '\oiint\limits_{%}^{%} %%'; Hint: 'Surface Integral'),
    (Size: meizNormal; ExprType: metContourIntegral; Command: '\oiiint %%'; Hint: 'Volume Integral'),
    (Size: meizNormal; ExprType: metContourIntegral; Command: '\oiiint\nolimits_{%}^{%} %%'; Hint: 'Volume Integral'),
    (Size: meizNormal; ExprType: metContourIntegral; Command: '\oiiint\limits_{%}^{%} %%'; Hint: 'Volume Integral'),

  //Differential
    (Size: meizNormal; ExprType: metDifferential; Command: 'dx'; Hint: 'Differential'),
    (Size: meizNormal; ExprType: metDifferential; Command: 'dy'; Hint: 'Differential'),
    (Size: meizNormal; ExprType: metDifferential; Command: 'd\theta'; Hint: 'Differential'),

  //Summs
    (Size: meizNormal; ExprType: metSumms; Command: '\sum %%'; Hint: 'Summation'),
    (Size: meizNormal; ExprType: metSumms; Command: '\sum\limits_{%}^{%} %%'; Hint: 'Summation'),
    (Size: meizNormal; ExprType: metSumms; Command: '\sum\nolimits_{%}^{%} %%'; Hint: 'Summation'),
    (Size: meizNormal; ExprType: metSumms; Command: '\sum\limits_{%} %%'; Hint: 'Summation'),
    (Size: meizNormal; ExprType: metSumms; Command: '\sum\nolimits_{%} %%'; Hint: 'Summation'),

  //Prod and Coprods
    (Size: meizNormal; ExprType: metProds; Command: '\prod %%'; Hint: 'Product'),
    (Size: meizNormal; ExprType: metProds; Command: '\prod\limits_{%}^{%} %%'; Hint: 'Product'),
    (Size: meizNormal; ExprType: metProds; Command: '\prod\nolimits_{%}^{%} %%'; Hint: 'Product'),
    (Size: meizNormal; ExprType: metProds; Command: '\prod\limits_{%} %%'; Hint: 'Product'),
    (Size: meizNormal; ExprType: metProds; Command: '\prod\nolimits_{%} %%'; Hint: 'Product'),
    (Size: meizNormal; ExprType: metProds; Command: '\coprod %%'; Hint: 'Co-Product'),
    (Size: meizNormal; ExprType: metProds; Command: '\coprod\limits_{%}^{%} %%'; Hint: 'Co-Product'),
    (Size: meizNormal; ExprType: metProds; Command: '\coprod\nolimits_{%}^{%} %%'; Hint: 'Co-Product'),
    (Size: meizNormal; ExprType: metProds; Command: '\coprod\limits_{%} %%'; Hint: 'Co-Product'),
    (Size: meizNormal; ExprType: metProds; Command: '\coprod\nolimits_{%} %%'; Hint: 'Co-Product'),

  //Unions
    (Size: meizNormal; ExprType: metUnions; Command: '\bigcup %%'; Hint: 'Union'),
    (Size: meizNormal; ExprType: metUnions; Command: '\bigcup\limits_{%}^{%} %%'; Hint: 'Union'),
    (Size: meizNormal; ExprType: metUnions; Command: '\bigcup\nolimits_{%}^{%} %%'; Hint: 'Union'),
    (Size: meizNormal; ExprType: metUnions; Command: '\bigcup\limits_{%} %%'; Hint: 'Union'),
    (Size: meizNormal; ExprType: metUnions; Command: '\bigcup\nolimits_{%} %%'; Hint: 'Union'),
    (Size: meizNormal; ExprType: metUnions; Command: '\bigcap %%'; Hint: 'Intersection'),
    (Size: meizNormal; ExprType: metUnions; Command: '\bigcap\limits_{%}^{%} %%'; Hint: 'Intersection'),
    (Size: meizNormal; ExprType: metUnions; Command: '\bigcap\nolimits_{%}^{%} %%'; Hint: 'Intersection'),
    (Size: meizNormal; ExprType: metUnions; Command: '\bigcap\limits_{%} %%'; Hint: 'Intersection'),
    (Size: meizNormal; ExprType: metUnions; Command: '\bigcap\nolimits_{%} %%'; Hint: 'Intersection'),

  //Other Large Operators
    (Size: meizNormal; ExprType: metOtherLarge; Command: '\bigvee %%'; Hint: 'Vee'),
    (Size: meizNormal; ExprType: metOtherLarge; Command: '\bigvee\limits_{%}^{%} %%'; Hint: 'Vee'),
    (Size: meizNormal; ExprType: metOtherLarge; Command: '\bigvee\nolimits_{%}^{%} %%'; Hint: 'Vee'),
    (Size: meizNormal; ExprType: metOtherLarge; Command: '\bigvee\limits_{%} %%'; Hint: 'Vee'),
    (Size: meizNormal; ExprType: metOtherLarge; Command: '\bigvee\nolimits_{%} %%'; Hint: 'Vee'),
    (Size: meizNormal; ExprType: metOtherLarge; Command: '\bigwedge %%'; Hint: 'Wedge'),
    (Size: meizNormal; ExprType: metOtherLarge; Command: '\bigwedge\limits_{%}^{%} %%'; Hint: 'Wedge'),
    (Size: meizNormal; ExprType: metOtherLarge; Command: '\bigwedge\nolimits_{%}^{%} %%'; Hint: 'Wedge'),
    (Size: meizNormal; ExprType: metOtherLarge; Command: '\bigwedge\limits_{%} %%'; Hint: 'Wedge'),
    (Size: meizNormal; ExprType: metOtherLarge; Command: '\bigwedge\nolimits_{%} %%'; Hint: 'Wedge'),

  //Common Large Operators
    (Size: meizNormal; ExprType: metCommonLarge; Command: '\sum_{k} \left( \stack{n}{k} \right)'; Hint: 'Summation'),
    (Size: meizNormal; ExprType: metCommonLarge; Command: '\sum_{i=0}^{n} %%'; Hint: 'Summation'),
    (Size: meizNormal; ExprType: metCommonLarge; Command: '\prod_{k=0}^{n} A_k'; Hint: 'Product'),
    (Size: meizNormalSuperWide; ExprType: metCommonLarge; Command: '\sum_{ \stack{0 \leq i \leq m}{0<j<n} } P(i,j)'; Hint: 'Summation'),
    (Size: meizNormalSuperWide; ExprType: metCommonLarge; Command: '\bigcup_{n=1}^{m}( X_n \cap Y_n )'; Hint: 'Union'),

  //Bracket
    (Size: meizNormal; ExprType: metBracket; Command: '\left( %% \right)'; Hint: 'Brackets'),
    (Size: meizNormal; ExprType: metBracket; Command: '\left\[ %% \right\]'; Hint: 'Brackets'),
    (Size: meizNormal; ExprType: metBracket; Command: '\left\{ %% \right\}'; Hint: 'Brackets'),
    (Size: meizNormal; ExprType: metBracket; Command: '\left\langle %% \right\rangle'; Hint: 'Brackets'),
    (Size: meizNormal; ExprType: metBracket; Command: '\left\lfloor %% \right\rfloor'; Hint: 'Brackets'),
    (Size: meizNormal; ExprType: metBracket; Command: '\left\lceil %% \right\rceil'; Hint: 'Brackets'),
    (Size: meizNormal; ExprType: metBracket; Command: '\left| %% \right|'; Hint: 'Brackets'),
    (Size: meizNormal; ExprType: metBracket; Command: '\left\Vert %% \right\Vert'; Hint: 'Brackets'),
    (Size: meizNormal; ExprType: metBracket; Command: '\left[ %% \right['; Hint: 'Brackets'),
    (Size: meizNormal; ExprType: metBracket; Command: '\left] %% \right]'; Hint: 'Brackets'),
    (Size: meizNormal; ExprType: metBracket; Command: '\left] %% \right['; Hint: 'Brackets'),
    (Size: meizNormal; ExprType: metBracket; Command: '\left\llbracket %% \right\rrbracket'; Hint: 'Brackets'),

  //Bracket with Separator
    (Size: meizNormal; ExprType: metBracketWithSeparator; Command: '\left( %% \middle| % \right)'; Hint: 'Brackets with Separators'),
    (Size: meizNormal; ExprType: metBracketWithSeparator; Command: '\left\{ %% \middle| % \right\}'; Hint: 'Brackets with Separators'),
    (Size: meizNormal; ExprType: metBracketWithSeparator; Command: '\left\langle %% \middle| % \right\rangle'; Hint: 'Brackets with Separators'),
    (Size: meizNormal; ExprType: metBracketWithSeparator; Command: '\left\langle %% \middle| % \middle| % \right\rangle'; Hint: 'Brackets with Separators'),

  //Single Bracket
    (Size: meizNormal; ExprType: metSingleBracket; Command: '\left( %% \right.'; Hint: 'Single Bracket'),
    (Size: meizNormal; ExprType: metSingleBracket; Command: '\left. %% \right\}'; Hint: 'Single Bracket'),
    (Size: meizNormal; ExprType: metSingleBracket; Command: '\left\[ %% \right.'; Hint: 'Single Bracket'),
    (Size: meizNormal; ExprType: metSingleBracket; Command: '\left. %% \right\]'; Hint: 'Single Bracket'),
    (Size: meizNormal; ExprType: metSingleBracket; Command: '\left\{ %% \right.'; Hint: 'Single Bracket'),
    (Size: meizNormal; ExprType: metSingleBracket; Command: '\left. %% \right\}'; Hint: 'Single Bracket'),
    (Size: meizNormal; ExprType: metSingleBracket; Command: '\left\langle %% \right.'; Hint: 'Single Bracket'),
    (Size: meizNormal; ExprType: metSingleBracket; Command: '\left. %% \right\rangle'; Hint: 'Single Bracket'),
    (Size: meizNormal; ExprType: metSingleBracket; Command: '\left\lfloor %% \right.'; Hint: 'Single Bracket'),
    (Size: meizNormal; ExprType: metSingleBracket; Command: '\left. %% \right\rfloor'; Hint: 'Single Bracket'),
    (Size: meizNormal; ExprType: metSingleBracket; Command: '\left\lceil %% \right.'; Hint: 'Single Bracket'),
    (Size: meizNormal; ExprType: metSingleBracket; Command: '\left. %% \right\rceil'; Hint: 'Single Bracket'),
    (Size: meizNormal; ExprType: metSingleBracket; Command: '\left| %% \right.'; Hint: 'Single Bracket'),
    (Size: meizNormal; ExprType: metSingleBracket; Command: '\left. %% \right|'; Hint: 'Single Bracket'),
    (Size: meizNormal; ExprType: metSingleBracket; Command: '\left\Vert %% \right.'; Hint: 'Single Bracket'),
    (Size: meizNormal; ExprType: metSingleBracket; Command: '\left. %% \right\Vert'; Hint: 'Single Bracket'),
    (Size: meizNormal; ExprType: metSingleBracket; Command: '\left\llbracket %% \right.'; Hint: 'Single Bracket'),
    (Size: meizNormal; ExprType: metSingleBracket; Command: '\left. %% \right\rrbracket'; Hint: 'Single Bracket'),

  //Case and Stacks
    (Size: meizNormal; ExprType: metCaseAndStack; Command: '\case{ %% \\ % }'; Hint: 'Case (Two Conditions)'),
    (Size: meizNormal; ExprType: metCaseAndStack; Command: '\case{ %% \\ % \\ % }'; Hint: 'Case (Three Conditions)'),
    (Size: meizNormal; ExprType: metCaseAndStack; Command: '\stack{%%}{%}'; Hint: 'Stack Object'),
    (Size: meizNormal; ExprType: metCaseAndStack; Command: '\left(\stack{%%}{%}\right)'; Hint: 'Stack Object'),

  //Common Bracket
    (Size: meizNormalWide; ExprType: metCommonBracket; Command: 'f(x) = \case{ -x & x < 0 \\ x & x \geq 0 }'; Hint: 'Cases Example'),
    (Size: meizNormal; ExprType: metCommonBracket; Command: '\left( \stack{n}{k} \right)'; Hint: 'Binomial Coefficient'),
    (Size: meizNormal; ExprType: metCommonBracket; Command: '\left\langle \stack{n}{k} \right\rangle'; Hint: 'Binomial Coefficient'),

  //Trigonometric Functions
    (Size: meizNormal; ExprType: metTrigFunction; Command: '\sin %%'; Hint: 'Sine Function'),
    (Size: meizNormal; ExprType: metTrigFunction; Command: '\cos %%'; Hint: 'Cosine Function'),
    (Size: meizNormal; ExprType: metTrigFunction; Command: '\tan %%'; Hint: 'Tangent Function'),
    (Size: meizNormal; ExprType: metTrigFunction; Command: '\csc %%'; Hint: 'Cosecant Function'),
    (Size: meizNormal; ExprType: metTrigFunction; Command: '\sec %%'; Hint: 'Secant Function'),
    (Size: meizNormal; ExprType: metTrigFunction; Command: '\cot %%'; Hint: 'Cotangent Function'),

  //Inverse Functions
    (Size: meizNormal; ExprType: metInvFunction; Command: '\sin^{-1} %%'; Hint: 'Inverse Sine Function'),
    (Size: meizNormal; ExprType: metInvFunction; Command: '\cos^{-1} %%'; Hint: 'Inverse Cosine Function'),
    (Size: meizNormal; ExprType: metInvFunction; Command: '\tan^{-1} %%'; Hint: 'Inverse Tangent Function'),
    (Size: meizNormal; ExprType: metInvFunction; Command: '\csc^{-1} %%'; Hint: 'Inverse Cosecant Function'),
    (Size: meizNormal; ExprType: metInvFunction; Command: '\sec^{-1} %%'; Hint: 'Inverse Secant Function'),
    (Size: meizNormal; ExprType: metInvFunction; Command: '\cot^{-1} %%'; Hint: 'Inverse Cotangent Function'),

  //Hypoerbolic Functions
    (Size: meizNormal; ExprType: metHyperFunction; Command: '\sinh %%'; Hint: 'Hyperbolic Sine Function'),
    (Size: meizNormal; ExprType: metHyperFunction; Command: '\cosh %%'; Hint: 'Hyperbolic Cosine Function'),
    (Size: meizNormal; ExprType: metHyperFunction; Command: '\tanh %%'; Hint: 'Hyperbolic Tangent Function'),

  //Inverse Hypoerbolic Functions
    (Size: meizNormal; ExprType: metInvHyperFunction; Command: '\sinh^{-1} %%'; Hint: 'Inverse Hyperbolic Sine Function'),
    (Size: meizNormal; ExprType: metInvHyperFunction; Command: '\cosh^{-1} %%'; Hint: 'Inverse Hyperbolic Cosine Function'),
    (Size: meizNormal; ExprType: metInvHyperFunction; Command: '\tanh^{-1} %%'; Hint: 'Inverse Hyperbolic Tangent Function'),

  //Common Functions
    (Size: meizNormal; ExprType: metCommonFunction; Command: '\sin \theta'; Hint: 'Sin θ'),
    (Size: meizNormal; ExprType: metCommonFunction; Command: '\cos 2x'; Hint: 'Cos 2x'),
    (Size: meizNormalMedium; ExprType: metCommonFunction; Command: '\tan\theta = \frac{\sin\theta}{\cos\theta}'; Hint: 'Tangent formula'),

  //Accents
    (Size: meizSmall; ExprType: metAccent; Command: '\dot{ %% }'; Hint: 'Dot'),
    (Size: meizSmall; ExprType: metAccent; Command: '\ddot{ %% }'; Hint: 'Double Dot'),
    (Size: meizSmall; ExprType: metAccent; Command: '\dddot{ %% }'; Hint: 'Triple Dot'),
    (Size: meizSmall; ExprType: metAccent; Command: '\hat{ %% }'; Hint: 'Hat'),
    (Size: meizSmall; ExprType: metAccent; Command: '\check{ %% }'; Hint: 'Check'),
    (Size: meizSmall; ExprType: metAccent; Command: '\acute{ %% }'; Hint: 'Acute'),
    (Size: meizSmall; ExprType: metAccent; Command: '\grave{ %% }'; Hint: 'Grave'),
    (Size: meizSmall; ExprType: metAccent; Command: '\breve{ %% }'; Hint: 'Breve'),
    (Size: meizSmall; ExprType: metAccent; Command: '\tilde{ %% }'; Hint: 'Tilde'),
    (Size: meizSmall; ExprType: metAccent; Command: '\bar{ %% }'; Hint: 'Bar'),
    (Size: meizSmall; ExprType: metAccent; Command: '\doverbar{ %% }'; Hint: 'Double Overbar'),
    (Size: meizSmall; ExprType: metAccent; Command: '\overbrace{ %% }'; Hint: 'Overbrace'),
    (Size: meizSmall; ExprType: metAccent; Command: '\underbrace{ %% }'; Hint: 'Underbrace'),
    (Size: meizSmall; ExprType: metAccent; Command: '\overbrace{ %% }^{ % }'; Hint: 'Grouping Character Above'),
    (Size: meizSmall; ExprType: metAccent; Command: '\underbrace{ %% }_{ % }'; Hint: 'Grouping Character Below'),
    (Size: meizSmall; ExprType: metAccent; Command: '\overleftarrow{ %% }'; Hint: 'Leftwards Arrow Above'),
    (Size: meizSmall; ExprType: metAccent; Command: '\vec{ %% }'; Hint: 'Rightwards Arrow Above'),
    (Size: meizSmall; ExprType: metAccent; Command: '\overleftrightarrow{ %% }'; Hint: 'Right-Left Arrow Above'),
    (Size: meizSmall; ExprType: metAccent; Command: '\leftharpoonaccent{ %% }'; Hint: 'Leftwards Harpoon Above'),
    (Size: meizSmall; ExprType: metAccent; Command: '\rightharpoonaccent{ %% }'; Hint: 'Rightwards Harpoon Above'),

  //Boxed
    (Size: meizSmall; ExprType: metBoxed; Command: '\boxed{ %% }'; Hint: 'Boxed Formula Boxed Formula (Example)(With Placeholder)'),
    (Size: meizSmallWide; ExprType: metBoxed; Command: '\boxed{a^2 + b^2 = c^2}'; Hint: ''),

  //Over and Underbars
    (Size: meizSmall; ExprType: metOverUnderbar; Command: '\overline{ %% }'; Hint: 'Overbar'),
    (Size: meizSmall; ExprType: metOverUnderbar; Command: '\underline{ %% }'; Hint: 'Underbar'),

  //Common Accent Objects
    (Size: meizSmallWide; ExprType: metCommonAccent; Command: '\overline{ ABC }'; Hint: 'ABC With Overbar'),
    (Size: meizSmallWide; ExprType: metCommonAccent; Command: '\overline{ x \oplus y }'; Hint: 'x XOR y With Overbar'),

  //Limit and Logs
    (Size: meizNormal; ExprType: metLimitAndLog; Command: '\log_{%} %%'; Hint: 'Logarithm'),
    (Size: meizNormal; ExprType: metLimitAndLog; Command: '\log %%'; Hint: 'Logarithm'),
    (Size: meizNormal; ExprType: metLimitAndLog; Command: '\lim\limits_{%} %%'; Hint: 'Limit'),
    (Size: meizNormal; ExprType: metLimitAndLog; Command: '\min\limits_{%} %%'; Hint: 'Minimum'),
    (Size: meizNormal; ExprType: metLimitAndLog; Command: '\max\limits_{%} %%'; Hint: 'Maximum'),
    (Size: meizNormal; ExprType: metLimitAndLog; Command: '\ln %%'; Hint: 'Natural Logarithm'),

  //Common Limit and Logs
    (Size: meizNormalMedium; ExprType: metCommonLimitAndLog; Command: '\lim\limits_{n \to \infty} {\left(1 + \frac{1}{n} \right)}^n'; Hint: 'Limit Example'),
    (Size: meizNormalMedium; ExprType: metCommonLimitAndLog; Command: '\max\limits_{0 \leq x \leq 1} x e^{-x^2}'; Hint: 'Maximum Example'),

  //Operator Basic
    (Size: meizSmall; ExprType: metOperatorBasic; Command: '\coloneq'; Hint: 'Colon Equal'),
    (Size: meizSmall; ExprType: metOperatorBasic; Command: '=='; Hint: 'Equal Equal'),
    (Size: meizSmall; ExprType: metOperatorBasic; Command: '+='; Hint: 'Plus Equal'),
    (Size: meizSmall; ExprType: metOperatorBasic; Command: '-='; Hint: 'Minus Equal'),
    (Size: meizSmall; ExprType: metOperatorBasic; Command: '\eqdef'; Hint: 'Equal To By Definition'),
    (Size: meizSmall; ExprType: metOperatorBasic; Command: '\measeq'; Hint: 'Measured By'),
    (Size: meizSmall; ExprType: metOperatorBasic; Command: '\triangleq'; Hint: 'Delta Equal To'),

  //Operator Structure
    (Size: meizSmall; ExprType: metOperatorStruct; Command: '\xleftarrow[ %% ]{}'; Hint: 'Leftwards Arrow Above'),
    (Size: meizSmall; ExprType: metOperatorStruct; Command: '\xrightarrow[ %% ]{}'; Hint: 'Rightwards Arrow Above'),
    (Size: meizSmall; ExprType: metOperatorStruct; Command: '\xleftarrow{ %% }'; Hint: 'Leftwards Arrow Below'),
    (Size: meizSmall; ExprType: metOperatorStruct; Command: '\xrightarrow{ %% }'; Hint: 'Rightwards Arrow Below'),
    (Size: meizSmall; ExprType: metOperatorStruct; Command: '\xLeftarrow[ %% ]{}'; Hint: 'Leftwards Arrow Above'),
    (Size: meizSmall; ExprType: metOperatorStruct; Command: '\xRightarrow[ %% ]{}'; Hint: 'Rightwards Arrow Above'),
    (Size: meizSmall; ExprType: metOperatorStruct; Command: '\xLeftarrow{ %% }'; Hint: 'Leftwards Arrow Below'),
    (Size: meizSmall; ExprType: metOperatorStruct; Command: '\xRightarrow{ %% }'; Hint: 'Rightwards Arrow Below'),
    (Size: meizSmall; ExprType: metOperatorStruct; Command: '\xleftrightarrow[ %% ]{}'; Hint: 'Right-Left Arrow Above'),
    (Size: meizSmall; ExprType: metOperatorStruct; Command: '\xleftrightarrow{ %% }'; Hint: 'Right-Left Arrow Below'),
    (Size: meizSmall; ExprType: metOperatorStruct; Command: '\xLeftrightarrow[ %% ]{}'; Hint: 'Right-Left Arrow Above'),
    (Size: meizSmall; ExprType: metOperatorStruct; Command: '\xLeftrightarrow{ %% }'; Hint: 'Right-Left Arrow Below'),

  //Common Operator
    (Size: meizSmallWide; ExprType: metCommonOperator; Command: '\xrightarrow{yields}'; Hint: 'Yields'),
    (Size: meizSmall; ExprType: metCommonOperator; Command: '\xrightarrow{\Delta}'; Hint: 'Delta Yields'),

  //Empty Matrixes
    (Size: meizNormal; ExprType: metEmptyMatrix; Command: '\matrix[..]{ %% & % }'; Hint: '1x2 Empty Matrix'),
    (Size: meizNormalWide; ExprType: metEmptyMatrix; Command: '\matrix[..]{ %% \\ % }'; Hint: '2x1 Empty Matrix'),
    (Size: meizNormalWide; ExprType: metEmptyMatrix; Command: '\matrix[..]{ %% & % & % }'; Hint: '1x3 Empty Matrix'),
    (Size: meizNormalWide; ExprType: metEmptyMatrix; Command: '\matrix[..]{ %% \\ % \\ % }'; Hint: '3x1 Empty Matrix'),
    (Size: meizNormalWide; ExprType: metEmptyMatrix; Command: '\matrix[..]{ %% & % \\ % & % }'; Hint: '2x2 Empty Matrix'),
    (Size: meizNormalWide; ExprType: metEmptyMatrix; Command: '\matrix[..]{ %% & % & % \\ % & % & % }'; Hint: '2x3 Empty Matrix'),
    (Size: meizNormalWide; ExprType: metEmptyMatrix; Command: '\matrix[..]{ %% & % \\ % & % \\ % & % }'; Hint: '3x2 Empty Matrix'),
    (Size: meizNormalWide; ExprType: metEmptyMatrix; Command: '\matrix[..]{ %% & % & % \\ % & % & % \\ % & % & % }'; Hint: '3x3 Empty Matrix'),

  //Dots
    (Size: meizNormal; ExprType: metDots; Command: '\cdots'; Hint: 'Midline Dots'),
    (Size: meizNormal; ExprType: metDots; Command: '\ldots'; Hint: 'Baseline Dots'),
    (Size: meizNormal; ExprType: metDots; Command: '\vdots'; Hint: 'Vertical Dots'),
    (Size: meizNormal; ExprType: metDots; Command: '\ddots'; Hint: 'Diagonal Dots'),

  //Indentity Matrixes
    (Size: meizNormalWide; ExprType: metIndentityMatrix; Command: '\matrix[..]{ 1 & 0 \\ 0 & 1 }'; Hint: '2x2 Identity Matrix'),
    (Size: meizNormalWide; ExprType: metIndentityMatrix; Command: '\matrix[..]{ 1 &  \\  & 1 }'; Hint: '2x2 Identity Matrix'),
    (Size: meizNormalWide; ExprType: metIndentityMatrix; Command: '\matrix[..]{ 1 & 0 & 0 \\ 0 & 1 & 0 \\ 0 & 0 & 1 }'; Hint: '3x3 Identity Matrix'),
    (Size: meizNormalWide; ExprType: metIndentityMatrix; Command: '\matrix[..]{ 1 &  &  \\  & 1 &  \\  &  & 1 }'; Hint: '3x3 Identity Matrix'),

  //Matrixes With Brackets
    (Size: meizNormal; ExprType: metMatrixWithBrackets; Command: '\matrix{ %% & % \\ % & % }'; Hint: 'Empty Matrix with Brackets'),
    (Size: meizNormal; ExprType: metMatrixWithBrackets; Command: '\matrix[\[\]]{ %% & % \\ % & % }'; Hint: 'Empty Matrix with Brackets'),
    (Size: meizNormal; ExprType: metMatrixWithBrackets; Command: '\matrix[\vert\vert]{ %% & % \\ % & % }'; Hint: 'Empty Matrix with Brackets'),
    (Size: meizNormal; ExprType: metMatrixWithBrackets; Command: '\matrix[\Vert\Vert]{ %% & % \\ % & % }'; Hint: 'Empty Matrix with Brackets'),

  //Sparse Matrixes
    (Size: meizNormalWide; ExprType: metSparseMatrix; Command: '\matrix{ %% & \cdots & % \\ \vdots & \ddots & \vdots \\ % & \cdots & % }'; Hint: 'Sparse Matrix'),
    (Size: meizNormalWide; ExprType: metSparseMatrix; Command: '\matrix[\[\]]{ %% & \cdots & % \\ \vdots & \ddots & \vdots \\ % & \cdots & % }'; Hint: 'Sparse Matrix')
  );

function GetSymbolSize(APPI: integer): integer; inline;
function GetExpressionSize(APPI: integer; ASize: TMathExpressionImageSize): TPoint; inline;
function HavePlaceholder(AString: string): boolean;
function GetMathSymbolType(AValue: TMathSymbolType): string;
function GetMathExpressionType(AValue: TMathExpressionType): string;

implementation

function GetSymbolSize(APPI: integer): integer;
begin
  if APPI = 96 then Result := 24
  else if APPI = 120 then Result := 30
  else if APPI = 144 then Result := 36
  else if APPI = 192 then Result := 48
  else Result := 24;
end;

function GetExpressionSize(APPI: integer; ASize: TMathExpressionImageSize): TPoint;
begin
  case ASize of
    meizNormal:
    begin
      Result.X := MulDiv(APPI, 3, 4);
      Result.Y := APPI;
    end;

    meizNormalMedium:
    begin
      Result.X := MulDiv(APPI, 9, 8) + MulDiv(APPI, 1, 24);
      Result.Y := APPI;
    end;

    meizNormalWide:
    begin
      Result.X := MulDiv(APPI, 6, 4) + MulDiv(APPI, 1, 16);
      Result.Y := APPI;
    end;

    meizNormalSuperWide:
    begin
      Result.X := MulDiv(APPI, 15, 8) + MulDiv(APPI, 3, 32);
      Result.Y := APPI;
    end;

    meizSmall:
    begin
      Result.X := APPI div 2;
      Result.Y := MulDiv(APPI, 3, 4);
    end;

    meizSmallWide:
    begin
      Result.X := APPI + MulDiv(APPI, 1, 16);
      Result.Y := MulDiv(APPI, 3, 4);
    end;

  else
    Result.X := MulDiv(APPI, 3, 4);
    Result.Y := APPI;
  end;
end;

function HavePlaceholder(AString: string): boolean;
begin
  Result := Pos('%', AString) > 0;
end;

function GetMathSymbolType(AValue: TMathSymbolType): string;
begin
  case AValue of
    mstBasic: Result := 'Basic Math';
    mstGreekLower: Result := 'Greek Lowercase';
    mstGreekUpper: Result := 'Greek Uppercase';
    mstLetterLike: Result := 'Letter-Like Symbols';
    mstBinaryOperators: Result := 'Common Binary Operators';
    mstRelationOperators: Result := 'Common Relational Operators';
    mstNaryOperators: Result := 'Basic N-ary Operators';
    mstAdvBinaryOperators: Result := 'Advanced Binary Operators';
    mstAdvRelationOperators: Result := 'Advanced Relational Operators';
    mstArrows: Result := 'Arrows';
    mstNegativeRelations: Result := 'Negated Relations';
    mstScripts: Result := 'Scripts';
    mstFrankturs: Result := 'Frakturs';
    mstDoubleStruck: Result := 'Double-Struck';
    mstGeometry: Result := 'Geometry';
  else
    Result := '';
  end;
end;

function GetMathExpressionType(AValue: TMathExpressionType): string;
begin
  case AValue of
    metFraction: Result := 'Fraction';
    metCommonFraction: Result := 'Common Fraction';
    metScript: Result := 'Subscripts and Superscripts';
    metCommonScript: Result := 'Common Subscripts and Superscripts';
    metRadical: Result := 'Radicals';
    metCommonRadical: Result := 'Common Radicals';
    metIntegral: Result := 'Integrals';
    metContourIntegral: Result := 'Contour Integrals';
    metDifferential: Result := 'Differentials';
    metSumms: Result := 'Summations';
    metProds: Result := 'Products and Co-Products';
    metUnions: Result := 'Unions and Intersections';
    metOtherLarge: Result := 'Other Large Operators';
    metCommonLarge: Result := 'Common Large Operators';
    metBracket: Result := 'Brackets';
    metBracketWithSeparator: Result := 'Brackets with Separators';
    metSingleBracket: Result := 'Single Brackets';
    metCaseAndStack: Result := 'Case and Stacks';
    metCommonBracket: Result := 'Common Brackets';
    metTrigFunction: Result := 'Trigonometric Functions';
    metInvFunction: Result := 'Inverse Functions';
    metHyperFunction: Result := 'Hyperbolic Functions';
    metInvHyperFunction: Result := 'Inverse Hyperbolic Functions';
    metCommonFunction: Result := 'Common Functions';
    metAccent: Result := 'Accents';
    metBoxed: Result := 'Boxed Formulas';
    metOverUnderbar: Result := 'Overbars and Underbars';
    metCommonAccent: Result := 'Common Accent Objects';
    metLimitAndLog: Result := 'Functions';
    metCommonLimitAndLog: Result := 'Common Functions';
    metOperatorBasic: Result := 'Basic Operators';
    metOperatorStruct: Result := 'Operator Structures';
    metCommonOperator: Result := 'Common Operator Structures';
    metEmptyMatrix: Result := 'Empty Matrices';
    metDots: Result := 'Dots';
    metIndentityMatrix: Result := 'Identity  Matrices';
    metMatrixWithBrackets: Result := 'Matrices with Brackets';
    metSparseMatrix: Result := 'Sparse Matrices';
  else
    Result := '';
  end;
end;

end.
